@@ -68,17 +68,27 @@ es_extendBuf(es_str_t **ps, es_size_t minNeeded)
 	int r = 0;
 	es_str_t *s = *ps;
 	es_size_t newSize;
+	es_size_t newAlloc;
 
 	ASSERT_STR(s);
-	newSize = 2 * s->lenBuf;
 	/* first compute the new size needed */
 	if(minNeeded > s->lenBuf) {
 		newSize = s->lenBuf + minNeeded;
 	} else {
 		newSize = 2 * s->lenBuf;
 	}
+	if(newSize < minNeeded) { /* overflow? */
+		r = ENOMEM;
+		goto done;
+	}
 
-	if((s = (es_str_t*) realloc(s, newSize + sizeof(es_str_t))) == NULL) {
+	newAlloc = newSize + sizeof(es_str_t);
+	if(newAlloc < newSize) { /* overflow? */
+		r = ENOMEM;
+		goto done;
+	}
+
+	if((s = (es_str_t*) realloc(s, newAlloc)) == NULL) {
 		r = errno;
 		goto done;
 	}
@@ -102,6 +112,10 @@ es_newStr(es_size_t lenhint)
 	if(lenhint & 0x07)
 		lenhint = lenhint - (lenhint & 0x07) + 8;
 
+	if(sizeof(es_str_t) + lenhint < lenhint) { /* overflow? */
+		s = NULL;
+		goto done;
+	}
 	if((s = malloc(sizeof(es_str_t) + lenhint)) == NULL)
 		goto done;
 
@@ -120,7 +134,6 @@ es_str_t*
 es_newStrFromCStr(const char *cstr, es_size_t len)
 {
 	es_str_t *s;
-	assert(strlen(cstr) == len);
 	
 	if((s = es_newStr(len)) == NULL) goto done;
 	memcpy(es_getBufAddr(s), cstr, len);
@@ -153,7 +166,15 @@ es_newStrFromNumber(long long num)
 	int i,j;
 	char minus = '\0';
 	es_str_t *s;
+	long long upperBorder = -9223372036854775807LL;
+	--upperBorder; /* handle number in C90 and newer modes */
 	
+	/* handle border case */
+	if(num == upperBorder) {
+		s = es_newStrFromCStr("-9223372036854775808", 20);
+		goto done;
+	}
+
 	if (num < 0) {
 	    minus = '-';
 	    num = -num;
@@ -186,6 +207,10 @@ es_newStrFromSubStr(es_str_t *str, es_size_t start, es_size_t len)
 {
 	es_str_t *s;
 	
+	if(start+len < start) {
+		s = NULL;
+		goto done;
+	}
 	if((s = es_newStr(len)) == NULL) goto done;
 
 	if(start > es_strlen(str))
@@ -212,7 +237,7 @@ es_deleteStr(es_str_t *s)
 
 
 int
-es_strbufcmp(es_str_t *s, unsigned char *buf, es_size_t lenBuf)
+es_strbufcmp(es_str_t *s, const unsigned char *buf, es_size_t lenBuf)
 {
 	int r;
 	es_size_t i;
@@ -245,7 +270,7 @@ es_strbufcmp(es_str_t *s, unsigned char *buf, es_size_t lenBuf)
  * call, so change propagation is easy ;)
  */
 int
-es_strcasebufcmp(es_str_t *s, unsigned char *buf, es_size_t lenBuf)
+es_strcasebufcmp(es_str_t *s, const unsigned char *buf, es_size_t lenBuf)
 {
 	int r;
 	es_size_t i;
@@ -418,7 +443,7 @@ done:	return r;
 
 
 int
-es_addChar(es_str_t **ps, unsigned char c)
+es_addChar(es_str_t **ps, const unsigned char c)
 {
 	int r = 0;
 
@@ -435,7 +460,7 @@ done:
 
 
 int
-es_addBuf(es_str_t **ps1, char *buf, es_size_t lenBuf)
+es_addBuf(es_str_t **ps1, const char *buf, es_size_t lenBuf)
 {
 	int r;
 	es_size_t newlen;
@@ -448,6 +473,10 @@ es_addBuf(es_str_t **ps1, char *buf, es_size_t lenBuf)
 	}
 
 	newlen = s1->lenStr + lenBuf;
+	if(newlen != (size_t) s1->lenStr + (size_t) lenBuf) {
+		r = ENOMEM;
+		goto done;
+	}
 	if(s1->lenBuf < newlen) {
 		/* we need to extend */
 		if((r = es_extendBuf(ps1, newlen - s1->lenBuf)) != 0) goto done;
@@ -465,12 +494,13 @@ done:
 
 
 char *
-es_str2cstr(es_str_t *s, char *nulEsc)
+es_str2cstr(es_str_t *s, const char *nulEsc)
 {
 	char *cstr;
 	es_size_t lenEsc;
 	int nbrNUL;
-	es_size_t i, iDst;
+	es_size_t i;
+	size_t iDst;
 	unsigned char *c;
 
 	/* detect number of NULs inside string */
@@ -498,7 +528,7 @@ es_str2cstr(es_str_t *s, char *nulEsc)
 			if(c[i] == 0x00) {
 				if(lenEsc == 1) {
 					cstr[iDst++] = *nulEsc;
-				} else {
+				} else if(lenEsc > 1) {
 					memcpy(cstr + iDst, nulEsc, lenEsc);
 					iDst += lenEsc;
 				}
@@ -558,7 +588,7 @@ es_str2num_hex(es_str_t *s, int *bSuccess)
 
 	i = 0;
 	num = 0;
-	c = es_getBufAddr(s);
+	c = es_getBufAddr(s) + 2;
 	while(i < s->lenStr && isxdigit(c[i])) {
 		if(isdigit(c[i]))
 			num = num * 16 + c[i] - '0';
@@ -584,7 +614,6 @@ es_str2num(es_str_t *s, int *bSuccess)
 		goto done;
 	}
 
-	num = 0;
 	c = es_getBufAddr(s);
 	if(c[0] == '-') {
 		num = -es_str2num_dec(s, 1, bSuccess);
@@ -666,6 +695,11 @@ doUnescape(unsigned char *c, es_size_t lenStr, es_size_t *iSrc, es_size_t iDst)
 			c[iDst] = '\\';
 			break;
 		case 'x':
+			if((*iSrc)+1 == lenStr) {
+				/* just end run, leave as is */
+				*iSrc += 1;
+				goto done;
+			}
 			if(    (*iSrc)+2 == lenStr
 			   || !isxdigit(c[(*iSrc)+1])
 			   || !isxdigit(c[(*iSrc)+2])) {
@@ -690,6 +724,7 @@ doUnescape(unsigned char *c, es_size_t lenStr, es_size_t *iSrc, es_size_t iDst)
 		/* regular character */
 		c[iDst] = c[*iSrc];
 	}
+done:	return;
 }
 
 void
